/*
 * Copyright 2013 Samsung Electronics Co., Ltd
 *
 *
 * Contact: Minsu Seo <minsu15.seo@samsung.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/xattr.h>
#include <unistd.h>
#include <string.h>
#include <dirent.h>
#include <glib.h>
#include <dlog.h>
#include <fontconfig/fontconfig.h>
#include <Elementary.h>
#include <pkgmgr-info.h>
#include <pkgmgr_installer_info.h>
#include "system_settings.h"

#define FONT_SERVICE_TAG 	 	"FONT_SERVICE"
#define DEBUG_LOG(frmt, args...)		\
	do { SLOG(LOG_DEBUG,FONT_SERVICE_TAG, "[font_service] %s: "frmt"\n",\
	__func__, ##args);} while (0)
#define DEBUG_WARNING(frmt, args...)		\
	do { SLOG(LOG_WARN,FONT_SERVICE_TAG, "[font_service] %s: "frmt"\n",\
	__func__, ##args);} while (0)
#define DEBUG_ERROR(frmt, args...)		\
	do { SLOG(LOG_ERROR,FONT_SERVICE_TAG, "[font_service] %s: "frmt"\n",\
	__func__, ##args);} while (0)
#define DEBUG_ERROR_ERRNO(frmt, args...)		\
	do { \
		char error_string[255] = { 0 }; \
		strerror_r(errno, error_string, sizeof(error_string)); \
		SLOG(LOG_ERROR,FONT_SERVICE_TAG, "[font_service] %s: "frmt": %s\n",\
	__func__, ##args, error_string);} while (0)

static const char *PARENT_PATH = "/opt/share/fonts";
static const char *DOWNLOAD_PATH = "/opt/share/fonts/download";
static const char *PRELOADED_PATH = "/opt/share/fonts/preloaded";

#define MAX_FILE_LEN 4096
#define APP_OWNER_ID 5001
#define APP_GROUP_ID 100

static int make_dir(const char *path);
static int symbolic_link(const char *srcpath, const char *destpath);
static int do_install(const char *parent, const char *appid, const char *rootpath, pkgmgrinfo_pkginfo_h handle);
static int do_uninstall(const char *deletedir);
static int move_path(const char *srcpath, const char *destpath);
static const char* check_preloaded(const char *app_root_path);
#ifdef CHECK_FAMILY_NAME
static int check_family_name(const char *srcpath);
#endif

static const char* check_preloaded(const char *app_root_path)
{
	char tpk_path[MAX_FILE_LEN];
	char wgt_path[MAX_FILE_LEN];

	snprintf(tpk_path, sizeof(tpk_path), "%s/preloaded", app_root_path);
	snprintf(wgt_path, sizeof(wgt_path), "%s/res/wgt/preloaded", app_root_path);

	if ((access(tpk_path, F_OK) == 0) || (access(wgt_path, F_OK) == 0))
	{
		return PRELOADED_PATH;
	}

	return DOWNLOAD_PATH;
}

static int make_dir(const char *path)
{
	int ret = 0;

	if (access(path, F_OK) == 0)
	{
		return ret;
	}
	else
	{
		ret = mkdir(path, 0755);
		if (ret < 0)
		{
			DEBUG_ERROR("make current directory %s is failed \n",path);
			return ret;
		}

		ret = chown(path, getuid(), APP_GROUP_ID);
		if (ret < 0)
		{
			DEBUG_ERROR("chown is failed %s\n",path);
			return ret;
		}

		ret = chmod(path, 0755);
		if (ret < 0)
		{
			DEBUG_ERROR("chmod is failed %s\n",path);
			rmdir(path);
			return ret;
		}
	}

	return ret;
}

static int symbolic_link(const char *srcpath, const char *destpath)
{
	DIR *d;
	struct dirent *e;
	int ret = 0;

	d = opendir(srcpath);
	while ((e = readdir (d)))
	{
		if (e->d_name[0] != '.' && strlen((char *)e->d_name) < MAX_FILE_LEN)
		{
			char srcdir[MAX_FILE_LEN];
			char destdir[MAX_FILE_LEN];
			struct stat	  statb;

			if (strlen (srcpath) + strlen ((char *)e->d_name) + 2 >= MAX_FILE_LEN )
			{
				DEBUG_ERROR("srcdir length is not available \n");
				goto FAIL;
			}

			if (strlen (destpath) + strlen ((char *)e->d_name) + 2 >= MAX_FILE_LEN )
			{
				DEBUG_ERROR("destdir length is not available \n");
				goto FAIL;
			}

			snprintf(srcdir, sizeof(srcdir), "%s/%s", srcpath, (char *)e->d_name);
			snprintf(destdir, sizeof(destdir), "%s/%s", destpath, (char *)e->d_name);
			if (stat (srcdir, &statb) == -1)
			{
				DEBUG_ERROR("stat %s is failed \n",srcdir);
				goto FAIL;
			}

			if (S_ISDIR (statb.st_mode))
			{
				if (make_dir(destdir)<0)
				{
					DEBUG_ERROR("make current directory %s is failed \n",destdir);
					goto FAIL;
				}
				ret = symbolic_link(srcdir,destdir);
				if (ret < 0)
				{
					DEBUG_ERROR("symlink is failed \n");
					goto FAIL;
				}
				continue;
			}
			DEBUG_LOG("srcdir =%s\n",(char *) srcdir);
			DEBUG_LOG("destdir =%s\n",(char *) destdir);
			DEBUG_LOG("file name =%s\n",(char *) e->d_name);

			ret = symlink((const char *)srcdir,(const char *)destdir);
			if (ret < 0)
			{
				DEBUG_ERROR("symlink is failed \n");
				goto FAIL;
			}
		}
	}
	closedir (d);
	return ret;

FAIL:
	closedir (d);
	return -1;
}

#define COPY_BUF_SIZE 16777216

static int copy_file(const char *srcpath, const char *destpath) {
	FILE *in = NULL, *out = NULL;
	char *buf = NULL;
	size_t len;

	if (!srcpath || !destpath)
	{
		DEBUG_ERROR("copyfile is failed. A given param is NULL ... srcpath[%s], destpath[%s]\n", srcpath, destpath);
		goto FAIL;
	}

	if (!strcmp(srcpath, destpath))
	{
		DEBUG_ERROR("copyfile is failed. The both path are same ... srcpath[%s], destpath[%s]\n", srcpath, destpath);
		goto FAIL;
	}

	if ((in  = fopen(srcpath, "rb")) == NULL)
	{
		DEBUG_ERROR_ERRNO("copyfile is failed. The srcpath[%s] is not opened with read permission. fopen()", srcpath);
		goto FAIL;
	}

	if ((out = fopen(destpath, "wb")) == NULL)
	{
		DEBUG_ERROR_ERRNO("copyfile is failed. The destpath[%s] is not opened with write permission. fopen()", destpath);
		goto FAIL;
	}

	if ((buf = (char *) malloc(COPY_BUF_SIZE)) == NULL)
	{
		DEBUG_ERROR_ERRNO("copyfile is failed. Memory allocation for copy buffer is failed. Request size [%d]. malloc()", COPY_BUF_SIZE);
		goto FAIL;
	}

	while ((len = fread(buf, sizeof(char), COPY_BUF_SIZE, in)) != 0)
	{
		if (fwrite(buf, sizeof(char), len, out) == 0)
		{
			/* Reads "len" string. But, it fails to write file.
			   We need to remove wrong result. */
			DEBUG_ERROR_ERRNO("copyfile is failed. fwrite fails to write. fwrite()");

			fclose(out);
			out = NULL;

			if (remove(destpath) == -1)
				DEBUG_ERROR_ERRNO("removing copied file with error is failed. remove()");

			goto FAIL;
		}
	}

	/* Copy file DONE! */
	if (in) fclose(in);
	if (out) fclose(out);
	if (buf) free(buf);
	return 0;

FAIL:
	if (in) fclose(in);
	if (out) fclose(out);
	if (buf) free(buf);
	return -1;
}

static int move_path(const char *srcpath, const char *destpath)
{
	DIR *d;
	struct dirent *e;
	int ret = 0;

	d = opendir(srcpath);
	while ((e = readdir(d)))
	{
		if (e->d_name[0] != '.' && strlen((char *)e->d_name) < MAX_FILE_LEN)
		{
			char srcdir[MAX_FILE_LEN];
			char destdir[MAX_FILE_LEN];
			struct stat statb;

			if (strlen(srcpath) + strlen((char *)e->d_name) + 2 >= MAX_FILE_LEN)
			{
				DEBUG_ERROR("srcdir length is not available \n");
				goto FAIL;
			}

			if (strlen(destpath) + strlen((char *)e->d_name) + 2 >= MAX_FILE_LEN)
			{
				DEBUG_ERROR("destdir length is not available \n");
				goto FAIL;
			}

			snprintf(srcdir, sizeof(srcdir), "%s/%s", srcpath, (char *)e->d_name);
			snprintf(destdir, sizeof(destdir), "%s/%s", destpath, (char *)e->d_name);

			if (stat(srcdir, &statb) == -1)
			{
				DEBUG_ERROR("stat %s is failed \n",srcdir);
				goto FAIL;
			}

			if (S_ISDIR(statb.st_mode))
			{
				if (make_dir(destdir) < 0)
				{
					DEBUG_ERROR("make current directory %s is failed \n", destdir);
					goto FAIL;
				}
				continue;
			}

			DEBUG_LOG("srcdir =%s\n", (char *)srcdir);
			DEBUG_LOG("destdir =%s\n", (char *)destdir);
			DEBUG_LOG("file name =%s\n", (char *)e->d_name);

			/* We need copy font file instead of rename() function.
			   To apply smack label of parent directory, we need create new file to the path. */
			ret = copy_file(srcdir, destdir);
			if (ret < 0)
			{
				DEBUG_ERROR("copy_file is failed \n");
				goto FAIL;
			}

			if (remove(srcdir) == -1)
				DEBUG_ERROR_ERRNO("removing src file[%s] is failed. remove()", srcdir);
		}
	}

	closedir(d);
	return ret;

FAIL:
	closedir(d);
	return -1;
}

#ifdef CHECK_FAMILY_NAME
static int check_family_name(const char *srcpath)
{
	FcObjectSet *os = NULL;
	FcFontSet* fs = NULL;
	FcPattern* pat = NULL;
	FcConfig* font_config = NULL;
	bool is_only_one_family = false;

	font_config = FcInitLoadConfigAndFonts();

	if(font_config == NULL) {
		DEBUG_ERROR("Failed: FcInitLoadConfigAndFonts");
		return -1;
	}

	pat = FcPatternCreate();

	os = FcObjectSetBuild(FC_FAMILY, FC_FILE, (char *) 0);

	if (os) {
		fs = FcFontList(font_config, pat, os);
		FcObjectSetDestroy(os);
		os = NULL;
	}

	if (pat) {
		FcPatternDestroy(pat);
		pat = NULL;
	}

	if (fs)
	{
		int j;
		char* only_one_family = NULL;

		for (j = 0; j < fs->nfont; j++)
		{
			FcChar8 *family = NULL;
			FcChar8 *file = NULL;

			if (FcPatternGetString(fs->fonts[j], FC_FILE, 0, &file) == FcResultMatch)
			{
				int font_path_len = strlen(srcpath);

					/* always shown for src path */
					if (strncmp((const char*)file, srcpath, font_path_len) == 0)
					{
								if (FcPatternGetString(fs->fonts[j], FC_FAMILY, 0, &family) != FcResultMatch)
								{
									DEBUG_ERROR("Family name is invalid");
									continue;
								}
								//DEBUG_ERROR("-------- FOUND FONT - family = %s", (char *)family);

								// first found family name
								if (only_one_family == NULL  && strlen((char *)family) > 0)
								{
									is_only_one_family = true;
									only_one_family = (char *)family;
									//DEBUG_ERROR("--------First FOUND FONT - family = %s", only_one_family);
								}
								else if (only_one_family != NULL  && strlen((char *)family) > 0)// after first
								{
										if (strcmp(only_one_family,  (char *)family) != 0)
										{
											DEBUG_ERROR("Not supported various font famliy. only one font famliy. %s %s", only_one_family, (char *)family);
											is_only_one_family = false;
											break;
										}
								}
								else
								{
									DEBUG_ERROR("invalid fonts");
									break;
								}
					}
			}
		}

		FcFontSetDestroy(fs);
		fs = NULL;
	}

	FcConfigDestroy(font_config);
	font_config = NULL;

	if (is_only_one_family == false)
	{
		//DEBUG_ERROR("Not supported various font famliy. only one font famliy");
		return -1;
	}

	return 0;
}
#endif

static int do_install(const char *parent, const char *appid, const char *rootpath, pkgmgrinfo_pkginfo_h handle)
{
	char destdir[MAX_FILE_LEN];
	char path[MAX_FILE_LEN];
	char *type = NULL;
	int ret = 0;

	if (strlen (parent) + strlen (appid) + 2 >= MAX_FILE_LEN )
	{
		DEBUG_ERROR("appid length is not available \n");
		return -1;
	}

	snprintf(destdir, sizeof(destdir), "%s/%s", parent, appid);
	ret = make_dir(destdir);
	if (ret < 0)
	{
		DEBUG_ERROR("make current directory %s is failed \n", destdir);
		goto FAIL;
	}

	if (strlen (rootpath) + 2 >= MAX_FILE_LEN )
	{
		DEBUG_ERROR("rootpath length is not available \n");
		goto FAIL;
	}

	ret = pkgmgrinfo_pkginfo_get_type(handle, &type);
	if (ret < 0)
	{
		DEBUG_ERROR("pkgmgrinfo_pkginfo_get_type is failed\n");
		goto FAIL;
	}

	snprintf(path, sizeof(path), "%s/shared/res", rootpath);

	if (!strcmp(type,"wgt"))
	{
		char srcpath[MAX_FILE_LEN];
		snprintf(srcpath, sizeof(srcpath), "%s/res/wgt/shared/res", rootpath);
		ret = move_path(srcpath, path);
	}

	if (ret < 0)
	{
		DEBUG_ERROR("move_path is failed\n");
		goto FAIL;
	}

	ret = symbolic_link(path, destdir);

	if (ret < 0)
	{
		DEBUG_ERROR("install is failed to [%s]\n", destdir);
		goto FAIL;
	}

#ifdef CHECK_FAMILY_NAME
	ret = check_family_name(destdir);

	if (ret < 0)
	{
		DEBUG_ERROR("Invaid font files\n");
		goto FAIL;
	}
#endif

	return ret;

FAIL:
	do_uninstall(destdir);
	return -1;
}


static int do_uninstall(const char *deletedir)
{
	DIR	*d;
	struct dirent *e;

	d = opendir(deletedir);
	while((e = readdir (d)))
	{
		if (e->d_name[0] != '.')
		{
			char destfile[MAX_FILE_LEN];
			struct stat	  statb;

			if (strlen (deletedir) + strlen ((char *) e->d_name) + 2 >= MAX_FILE_LEN )
			{
				DEBUG_ERROR("destfile length is not available \n");
				goto FAIL;
			}

			snprintf(destfile, sizeof(destfile), "%s/%s", deletedir, (char *)e->d_name);
			if (lstat (destfile, &statb) == -1)
			{
				DEBUG_ERROR("lstat %s is failed \n",destfile);
				goto FAIL;
			}

			if (S_ISDIR (statb.st_mode))
			{
				if (do_uninstall(destfile)<0)
					goto FAIL;
			}
			else if (unlink(destfile) < 0)
			{
				DEBUG_ERROR("unlink is failed %s\n",destfile);
				goto FAIL;
			}
			DEBUG_LOG("destfile =%s\n",destfile);
		}
	}

	closedir (d);

	if (rmdir(deletedir) < 0)
	{
		DEBUG_ERROR("rmdir is failed \n");
		return -1;
	}

	DEBUG_LOG("rmdir =%s\n",deletedir);
	return 0;

FAIL:
	closedir (d);
	return -1;
}


int COMMON_PKGMGR_PLUGIN_INSTALL(const char *pkgid, const char *appid, GList *list)
{
	int ret;
	pkgmgrinfo_pkginfo_h handle = NULL;
	const char *app_root_path = NULL;
	const char *dest_path = NULL;
	uid_t uid = 0;

	pkgmgr_installer_info_get_target_uid(&uid);
	ret = pkgmgrinfo_pkginfo_get_usr_pkginfo(pkgid, uid, &handle);

	if (ret < 0)
	{
		DEBUG_ERROR("pkgid[%s] handle get fail", pkgid);
		return -1;
	}

	ret = pkgmgrinfo_pkginfo_get_root_path(handle, (char **)&app_root_path);
	if (ret < 0)
	{
		DEBUG_ERROR("pkgid[%s] path get fail", pkgid);
		goto FAIL;
	}

	if (appid == NULL)
	{
		DEBUG_ERROR("appid is NULL \n");
		goto FAIL;
	}

	ret = make_dir(PARENT_PATH);

	if (ret < 0)
	{
		DEBUG_ERROR("make current directory is failed \n");
		goto FAIL;
	}

	dest_path = check_preloaded(app_root_path);

	if (make_dir(dest_path)<0)
	{
		DEBUG_ERROR("make current directory is failed \n");
		return -1;
	}

	ret = do_install(dest_path,appid, app_root_path, handle);
	if (ret < 0)
	{
		DEBUG_ERROR("font install is failed \n");
		goto FAIL;
	}

	pkgmgrinfo_pkginfo_destroy_pkginfo(handle);

	return ret;

FAIL:
	pkgmgrinfo_pkginfo_destroy_pkginfo(handle);
	return -1;
}


int COMMON_PKGMGR_PLUGIN_UPGRADE(const char *pkgid, const char *appid, GList *list)
{
	char deletedir[MAX_FILE_LEN];
	int ret = 0;
	pkgmgrinfo_pkginfo_h handle = NULL;
	const char* app_root_path = NULL;
	const char* dest_path = NULL;
	uid_t uid = 0;

	pkgmgr_installer_info_get_target_uid(&uid);
	ret = pkgmgrinfo_pkginfo_get_usr_pkginfo(pkgid, uid, &handle);

	if (ret < 0)
	{
		DEBUG_ERROR("pkgid[%s] handle get fail", pkgid);
		return -1;
	}

	ret = pkgmgrinfo_pkginfo_get_root_path(handle, (char **)&app_root_path);
	if (ret < 0)
	{
		DEBUG_ERROR("pkgid[%s] path get fail", pkgid);
		goto FAIL;
	}

	if (appid == NULL)
	{
		DEBUG_ERROR("appid is NULL \n");
		goto FAIL;
	}

	dest_path = check_preloaded(app_root_path);

	if (strlen (dest_path) + strlen (appid) + 2 >= MAX_FILE_LEN )
	{
		DEBUG_ERROR("appid length is not available \n");
		goto FAIL;
	}

	snprintf(deletedir, sizeof(deletedir), "%s/%s", dest_path, appid);

	if (access(deletedir, F_OK) == -1)
	{
		DEBUG_ERROR_ERRNO("dest directory(%s) is not exist", deletedir);
		goto FAIL;
	}

	ret = do_uninstall(deletedir);
	if (ret < 0)
	{
		DEBUG_ERROR("do_uninstall is failed \n");
		goto FAIL;
	}

	ret = make_dir(PARENT_PATH);
	if (ret < 0)
	{
		DEBUG_ERROR_ERRNO("make current directory(%s) is failed", PARENT_PATH);
		goto FAIL;
	}

	ret = make_dir(dest_path);
	if (ret < 0)
	{
		DEBUG_ERROR_ERRNO("make current directory(%s) is failed", dest_path);
		goto FAIL;
	}

	ret = do_install(dest_path, appid, app_root_path, handle);
	if (ret < 0)
	{
		DEBUG_ERROR("do_install is failed \n");
		goto FAIL;
	}

	pkgmgrinfo_pkginfo_destroy_pkginfo(handle);
	return ret;

FAIL:
	pkgmgrinfo_pkginfo_destroy_pkginfo(handle);
	return -1;
}



int COMMON_PKGMGR_PLUGIN_UNINSTALL(const char *pkgid, const char *appid, GList *list)
{
	char deletedir[MAX_FILE_LEN];
	FcObjectSet *os = NULL;
	FcPattern *pat = NULL;
	FcFontSet *fs = NULL;
	pkgmgrinfo_pkginfo_h handle = NULL;
	const char* app_root_path = NULL;
	const char *dest_path = NULL;
	int ret, deletedir_len;
	uid_t uid = 0;

	elm_init(0, NULL);

	if (appid == NULL)
	{
		DEBUG_ERROR("appid is NULL \n");
		return -1;
	}

	pkgmgr_installer_info_get_target_uid(&uid);
	ret = pkgmgrinfo_pkginfo_get_usr_pkginfo(pkgid, uid, &handle);

	if (ret < 0)
	{
		DEBUG_ERROR("pkgid[%s] handle get fail", pkgid);
		return -1;
	}

	ret = pkgmgrinfo_pkginfo_get_root_path(handle, (char **)&app_root_path);
	if (ret < 0)
	{
		DEBUG_ERROR("pkgid[%s] path get fail", pkgid);
		goto FAIL;
	}

	dest_path = check_preloaded(app_root_path);

	if (strlen (dest_path) + strlen (appid) + 2 >= MAX_FILE_LEN )
	{
		DEBUG_ERROR("appid length is not available \n");
		goto FAIL;
	}

	/* It must contain "/" character at end of the delete dir path.
	 * It prevents file path comparing issues when there are many similar path. */
	snprintf(deletedir, sizeof(deletedir), "%s/%s/", dest_path, appid);

	//check if current using font is same with uninstall font
	deletedir_len = strlen(deletedir);

	pat = FcPatternCreate();
	if (pat == NULL)
	{
		DEBUG_ERROR("FcPatternCreate is NULL \n");
		goto FAIL;
	}

	char *current_font_name = NULL;
	ret = system_settings_get_value_string(SYSTEM_SETTINGS_KEY_FONT_TYPE, &current_font_name);
	if (ret < 0)
	{
		DEBUG_ERROR("Get current font is failed \n");
		goto FAIL;
	}

	DEBUG_LOG("current_font_name =%s\n",current_font_name);

	FcInitLoadConfigAndFonts();
	FcPatternAddString (pat, FC_FAMILY,(FcChar8*)current_font_name);
	os = FcObjectSetBuild(FC_FAMILY, FC_FILE, (char *) 0);
	fs = FcFontList(NULL, pat, os);

	FcPatternDestroy(pat);
	pat = NULL;
	free(current_font_name);
	current_font_name = NULL;

	if (fs)
	{
		int j;

		for (j = 0; j < fs->nfont; j++)
		{
			FcChar8 *file = NULL;
			if (FcPatternGetString(fs->fonts[j], FC_FILE, 0, &file) == FcResultMatch)
			{
				DEBUG_LOG("file =%s, deletedir =%s\n", file, deletedir);
				if (strncmp((const char*)file, deletedir, deletedir_len) == 0)
				{
					DEBUG_LOG("change to default font\n");
					char *default_font_name = NULL;

					ret = system_settings_get_value_string(SYSTEM_SETTINGS_KEY_DEFAULT_FONT_TYPE, &default_font_name);
					if (ret < 0)
					{
						DEBUG_ERROR("Get default font is failed \n");
						goto FAIL;
					}

					DEBUG_LOG("default_font_name = %s \n",default_font_name);
					ret = system_settings_set_value_string(SYSTEM_SETTINGS_KEY_FONT_TYPE, default_font_name);
					if (ret < 0)
					{
						DEBUG_ERROR("Set default font is failed ret=%d \n",ret);
						free(default_font_name);
						break;
					}

					free(default_font_name);
					DEBUG_LOG("success change to default font\n");
					break;
				}
			}
		}
	}

	if (access(deletedir, F_OK) == -1)
	{
		DEBUG_ERROR_ERRNO("dest directory(%s) is not exist", deletedir);
		goto FAIL;
	}

	elm_shutdown();
	pkgmgrinfo_pkginfo_destroy_pkginfo(handle);

	return do_uninstall(deletedir);

FAIL:
	if (pat) FcPatternDestroy(pat);
	elm_shutdown();
	pkgmgrinfo_pkginfo_destroy_pkginfo(handle);

	return -1;
}

#if USE_META_DATA
int PKGMGR_MDPARSER_PLUGIN_INSTALL(const char *pkgid, const char *appid, GList *list)
{
	return COMMON_PKGMGR_PLUGIN_INSTALL(pkgid, appid, list);
}

int PKGMGR_MDPARSER_PLUGIN_UPGRADE(const char *pkgid, const char *appid, GList *list)
{
	return COMMON_PKGMGR_PLUGIN_UPGRADE(pkgid, appid, list);
}

int PKGMGR_MDPARSER_PLUGIN_UNINSTALL(const char *pkgid, const char *appid, GList *list)
{
	return COMMON_PKGMGR_PLUGIN_UNINSTALL(pkgid, appid, list);
}
#endif

int PKGMGR_CATEGORY_PARSER_PLUGIN_INSTALL(const char *pkgid, const char *appid, GList *list)
{
	return COMMON_PKGMGR_PLUGIN_INSTALL(pkgid, appid, list);
}

int PKGMGR_CATEGORY_PARSER_PLUGIN_UPGRADE(const char *pkgid, const char *appid, GList *list)
{
	return COMMON_PKGMGR_PLUGIN_UPGRADE(pkgid, appid, list);
}

int PKGMGR_CATEGORY_PARSER_PLUGIN_UNINSTALL(const char *pkgid, const char *appid, GList *list)
{
	return COMMON_PKGMGR_PLUGIN_UNINSTALL(pkgid, appid, list);
}


/* End of a file */
